@@ -6,15 +6,17 @@ struct _BrowserDescription {
 	gboolean in_path;
 };
 BrowserDescription possible_browsers[] = {
+	{ N_("Debian Sensible Browser"),	"sensible-browser",	"sensible-browser %s", FALSE, FALSE },
         { N_("Epiphany"), 		"epiphany",	"epiphany %s",		FALSE, FALSE },
         { N_("Galeon"), 		"galeon",	"galeon %s",		FALSE, FALSE },
         { N_("Encompass"), 		"encompass",	"encompass %s",		FALSE, FALSE },
-        { N_("Firebird/FireFox"),  "mozilla-firebird",	"mozilla-firebird %s",	FALSE, FALSE },
-        { N_("Mozilla/Netscape 6"), 	"mozilla-1.6",	"mozilla-1.6 %s",	FALSE, FALSE },
-        { N_("Mozilla/Netscape 6"), 	"mozilla",	"mozilla %s",		FALSE, FALSE },
+        { N_("Firebird"),       "mozilla-firebird",  "mozilla-firebird %s",  FALSE, FALSE },
+        { N_("Firefox"),  	"firefox",	"firefox %s",	FALSE, FALSE },
+        { N_("Mozilla"), 	"mozilla-1.6",	"mozilla-1.6 %s",	FALSE, FALSE },
+        { N_("Mozilla"), 	"mozilla",	"mozilla %s",		FALSE, FALSE },
         { N_("Netscape Communicator"), 	"netscape",	"netscape %s",		FALSE, FALSE },
         { N_("Konqueror"), 		"konqueror",	"konqueror %s",		FALSE, FALSE },
-        { N_("W3M Text Browser"),	"w3n",		"w3m %s",		TRUE,  FALSE },
+        { N_("W3M Text Browser"),	"w3m",		"w3m %s",		TRUE,  FALSE },
         { N_("Lynx Text Browser"),	"lynx",		"lynx %s",		TRUE,  FALSE },
         { N_("Links Text Browser") , 	"links",	"links %s",		TRUE,  FALSE }
 };
@@ -35,8 +37,9 @@ MailerDescription possible_mailers[] = {
 	{ N_("Evolution Mail Reader"),		"evolution-1.5",	"evolution-1.5 %s",	FALSE, FALSE, },
         { N_("Evolution Mail Reader"),		"evolution-1.4",	"evolution-1.4 %s",	FALSE, FALSE, },
         { N_("Evolution Mail Reader"),		"evolution",		"evolution %s",		FALSE, FALSE, },
-	{ N_("Balsa"),        			"balsa",		"balsa --compose=%s",	FALSE, FALSE },
+	{ N_("Balsa"),        			"balsa",		"balsa -m %s",		FALSE, FALSE },
 	{ N_("KMail"),        			"kmail",		"kmail %s",		FALSE, FALSE },
+	{ N_("Thunderbird"), 			"mozilla-thunderbird",	"mozilla-thunderbird -mail %s", FALSE, FALSE},
 	{ N_("Mozilla Mail"), 			"mozilla",		"mozilla -mail %s",	FALSE, FALSE},
         { N_("Mutt") , 	  			"mutt",			"mutt %s",		TRUE,  FALSE },
 
@@ -49,9 +52,11 @@ struct _TerminalDesciption {
 	gboolean in_path;
 };
 TerminalDescription possible_terminals[] = { 
-        { N_("Gnome Terminal"),		"gnome-terminal",	"-x", FALSE },
+        { N_("Debian Terminal Emulator"),	"x-terminal-emulator",	"-e", FALSE },
+        { N_("GNOME Terminal"),		"gnome-terminal",	"-x", FALSE },
         { N_("Standard XTerminal"),	"xterm",		"-e", FALSE },
         { N_("NXterm"),			"nxterm",		"-e", FALSE },
         { N_("RXVT"),			"rxvt",			"-e", FALSE },
+        { N_("aterm"),			"aterm",		"-e", FALSE },
         { N_("ETerm"),			"Eterm",		"-e", FALSE }
 };
