@@ -18,8 +18,7 @@
 #include "activate-settings-daemon.h"
 
 #define LABEL_DATA "gnome-keybinding-properties-label"
-#define KEY_THEME_KEY "/desktop/gnome/interface/gtk_key_theme"
-#define MAX_ELEMENTS_BEFORE_SCROLLING 8
+#define MAX_ELEMENTS_BEFORE_SCROLLING 10
 
 #if defined(__powerpc__) && defined (__linux__)
 #define USE_FBLEVEL
@@ -41,32 +40,37 @@ typedef struct
 
 const KeyListEntry desktop_key_list[] =
 {
+  { "/apps/gnome_settings_daemon/keybindings/help", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/power", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/sleep", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/screensaver", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/home", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/search", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/email", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/www", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/panel_run_dialog", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/panel_main_menu", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_screenshot", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_window_screenshot", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/volume_mute", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/volume_down", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/volume_up", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/power", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/eject", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/home", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/search", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/email", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/sleep", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/screensaver", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/help", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/www", ALWAYS_VISIBLE, 0 },
+  { "/apps/metacity/global_keybindings/run_command_terminal", ALWAYS_VISIBLE, 0 },
 #ifdef USE_FBLEVEL
   { "/apps/gnome_settings_daemon/keybindings/brightness_down", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/brightness_up", ALWAYS_VISIBLE, 0 },
 #endif
+  { NULL }
+};
+const KeyListEntry sounds_key_list[] =
+{
+  { "/apps/gnome_settings_daemon/keybindings/volume_mute", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_down", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_up", ALWAYS_VISIBLE, 0 },
   /* Other ones that need keysyms bindings */
   { "/apps/gnome_settings_daemon/keybindings/play", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/pause", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/stop", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/previous", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/next", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/eject", ALWAYS_VISIBLE, 0 },
   { NULL }
 };
 
@@ -171,48 +175,6 @@ get_real_model (GtkTreeView *tree_view)
   return submodel;
 }
 
-static void
-menu_item_activate (GtkWidget *menu_item,
-		    gpointer   unused)
-{
-  gchar *key_theme;
-  gchar *current_key_theme;
-  GConfClient *client;
-  GError *error = NULL;
-
-  client = gconf_client_get_default ();
-
-  key_theme = g_object_get_data (G_OBJECT (menu_item), LABEL_DATA);
-  g_return_if_fail (key_theme != NULL);
-
-  current_key_theme = gconf_client_get_string (client, KEY_THEME_KEY, &error);
-  if (current_key_theme && strcmp (current_key_theme, key_theme))
-    {
-      gconf_client_set_string (client, KEY_THEME_KEY, key_theme, NULL);
-    }
-}
-
-static GtkWidget *
-make_key_theme_menu_item (const gchar *key_theme)
-{
-  GtkWidget *retval;
-
-  if (!strcmp (key_theme, "Default"))
-    {
-      retval = gtk_menu_item_new_with_label (_("GNOME Default"));
-      g_object_set_data_full (G_OBJECT (retval), LABEL_DATA, g_strdup ("Default"), g_free);
-    }
-  else
-    {
-      retval = gtk_menu_item_new_with_label (key_theme);
-      g_object_set_data_full (G_OBJECT (retval), LABEL_DATA, g_strdup (key_theme), g_free);
-    }
-  g_signal_connect (G_OBJECT (retval), "activate", G_CALLBACK (menu_item_activate), NULL);
-  gtk_widget_show (retval);
-
-  return retval;
-}
-
 static GladeXML *
 create_dialog (void)
 {
@@ -594,6 +556,7 @@ reload_key_entries (gpointer wm_name, GladeXML *dialog)
   clear_old_model (dialog, WID ("shortcut_treeview"));
   
   append_keys_to_tree (dialog, _("Desktop"), desktop_key_list);
+  append_keys_to_tree (dialog, _("Sound"), sounds_key_list);
   
   if (strcmp((char *) wm_name, WM_COMMON_METACITY) == 0)
     {
@@ -610,75 +573,33 @@ key_entry_controlling_key_changed (GConfClient *client,
   reload_key_entries (wm_common_get_current_window_manager(), user_data);
 }
 
-static void
-key_theme_changed (GConfClient *client,
-		   guint        cnxn_id,
-		   GConfEntry  *entry,
-		   gpointer     user_data)
-{
-  GtkWidget *omenu = (GtkWidget *) user_data;
-  GtkWidget *menu;
-  GtkWidget *menu_item;
-  GConfValue *value;
-  const gchar *new_key_theme;
-  GList *list;
-  gint i = 0;
-
-  menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (omenu));
-  value = gconf_entry_get_value (entry);
-
-  g_return_if_fail (value != NULL);
-
-  new_key_theme = gconf_value_get_string (value);
-
-  for (list = GTK_MENU_SHELL (menu)->children; list; list = list->next, i++)
-    {
-      gchar *text;
-
-      menu_item = GTK_WIDGET (list->data);
-      text = g_object_get_data (G_OBJECT (menu_item), LABEL_DATA);
-      if (! strcmp (text, new_key_theme))
-	{
-	  if (gtk_option_menu_get_history (GTK_OPTION_MENU (omenu)) != i)
-	    gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), i);
-	  return;
-	}
-    }
-
-  /* We didn't find our theme.  Add it to our list. */
-  menu_item = make_key_theme_menu_item (new_key_theme);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);  
-  gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), i);
-}
-
-
 static gboolean
 cb_check_for_uniqueness (GtkTreeModel *model,
 			 GtkTreePath  *path,
 			 GtkTreeIter  *iter,
-			 gpointer      user_data)
+			 KeyEntry *new_key)
 {
-  KeyEntry *key_entry;
-  KeyEntry *tmp_key_entry;
+  KeyEntry *element;
 
-  key_entry = (KeyEntry *)user_data;
-  gtk_tree_model_get (key_entry->model, iter,
-		      KEYENTRY_COLUMN, &tmp_key_entry,
+  gtk_tree_model_get (new_key->model, iter,
+		      KEYENTRY_COLUMN, &element,
 		      -1);
 
-  if (tmp_key_entry != NULL &&
-      key_entry->keyval == tmp_key_entry->keyval &&
-      key_entry->mask   == tmp_key_entry->mask &&
-      key_entry->keycode == tmp_key_entry->keycode &&
-      /* be sure we don't claim a key is a dup of itself */
-      strcmp (key_entry->gconf_key, tmp_key_entry->gconf_key) != 0)
-    {
-      key_entry->editable = FALSE;
-      key_entry->gconf_key = tmp_key_entry->gconf_key;
-      key_entry->description = tmp_key_entry->description;
-      return TRUE;
-    }
-  return FALSE;
+  /* no conflict for : blanks, different modifiers, or ourselves */
+  if (element == NULL || new_key->mask != element->mask ||
+      !strcmp (new_key->gconf_key, element->gconf_key))
+    return FALSE;
+
+  if (new_key->keyval != 0) {
+      if (new_key->keyval != element->keyval)
+	  return FALSE;
+  } else if (element->keyval != 0 || new_key->keycode != element->keycode)
+    return FALSE;
+
+  new_key->editable = FALSE;
+  new_key->gconf_key = element->gconf_key;
+  new_key->description = element->description;
+  return TRUE;
 }
 
 static void
@@ -689,6 +610,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
 		       guint		      keycode,
                        gpointer               data)
 {
+  GConfClient *client;
   GtkTreeView *view = (GtkTreeView *)data;
   GtkTreeModel *model;
   GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
@@ -718,7 +640,9 @@ accel_edited_callback (GtkCellRendererText   *cell,
   tmp_key.editable = TRUE; /* kludge to stuff in a return flag */
 
   if (keyval != 0 || keycode != 0) /* any number of keys can be disabled */
-    gtk_tree_model_foreach (model, cb_check_for_uniqueness, &tmp_key);
+    gtk_tree_model_foreach (model,
+      (GtkTreeModelForeachFunc) cb_check_for_uniqueness,
+      &tmp_key);
 
   /* flag to see if the new accelerator was in use by something */
   if (!tmp_key.editable)
@@ -732,7 +656,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
         gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
                                 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
                                 GTK_MESSAGE_WARNING,
-                                GTK_BUTTONS_OK,
+                                GTK_BUTTONS_CANCEL,
                                 _("The shortcut \"%s\" is already used for:\n \"%s\"\n"),
                                 name,
                                 tmp_key.description ?
@@ -749,12 +673,14 @@ accel_edited_callback (GtkCellRendererText   *cell,
 
   str = binding_name (keyval, keycode, mask, FALSE);
 
-  gconf_client_set_string (gconf_client_get_default(),
+  client = gconf_client_get_default();
+  gconf_client_set_string (client,
                            key_entry->gconf_key,
                            str,
                            &err);
   g_free (str);
-  
+  g_object_unref (G_OBJECT (client));
+
   if (err != NULL)
     {
       GtkWidget *dialog;
@@ -778,6 +704,7 @@ accel_cleared_callback (GtkCellRendererText *cell,
 			const char          *path_string,
 			gpointer             data)
 {
+  GConfClient *client;
   GtkTreeView *view = (GtkTreeView *) data;
   GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
   KeyEntry *key_entry;
@@ -797,10 +724,13 @@ accel_cleared_callback (GtkCellRendererText *cell,
     return;
 
   /* Unset the key */
-  gconf_client_set_string (gconf_client_get_default(),
+  client = gconf_client_get_default();
+  gconf_client_set_string (client,
 			   key_entry->gconf_key,
 			   "disabled",
 			   &err);
+  g_object_unref (G_OBJECT (client));
+
   if (err != NULL)
     {
       GtkWidget *dialog;
@@ -819,49 +749,6 @@ accel_cleared_callback (GtkCellRendererText *cell,
     }
 }
 
-static void
-theme_changed_func (gpointer  uri,
-		    GladeXML *dialog)
-{
-  GConfClient *client;
-  GtkWidget *omenu;
-  GtkWidget *menu;
-  GtkWidget *menu_item;
-  GConfEntry *entry;
-  GList *key_theme_list;
-  GList *list;
-
-  client = gconf_client_get_default ();
-  key_theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2_KEYBINDING);
-
-  omenu = WID ("key_theme_omenu");
-  menu = gtk_menu_new ();
-  for (list = key_theme_list; list; list = list->next)
-    {
-      GnomeThemeInfo *info = list->data;
-
-      if (! info->has_keybinding)
-	continue;
-
-      menu_item = make_key_theme_menu_item (info->name);
-      if (!strcmp (info->name, "Default"))
-      /* Put default first, always */
-        gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
-      else
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-    }
-
-  gtk_widget_show (menu);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
-
-  /* Initialize the option menu */
-  entry = gconf_client_get_entry (client,
-				  KEY_THEME_KEY,
-				  NULL, TRUE, NULL);
-
-  key_theme_changed (client, 0, entry, omenu);
-}
-
 
 typedef struct
 {
@@ -931,50 +818,12 @@ static void
 setup_dialog (GladeXML *dialog)
 {
   GConfClient *client;
-  GList *key_theme_list;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkWidget *widget;
-  gboolean found_keys = FALSE;
-  GList *list;
 
   client = gconf_client_get_default ();
 
-  key_theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2_KEYBINDING);
-
-  for (list = key_theme_list; list; list = list->next)
-    {
-      GnomeThemeInfo *info = list->data;
-      if (info->has_keybinding)
-	{
-	  found_keys = TRUE;
-	  break;
-	}
-
-    }
-  if (! found_keys)
-    {
-      GtkWidget *msg_dialog = gtk_message_dialog_new (NULL, 0,
-						      GTK_MESSAGE_ERROR,
-						      GTK_BUTTONS_OK,
-						      _("Unable to find any keyboard themes.  This means your GTK+ "
-							"installation has been incompletely installed."));
-      gtk_dialog_run (GTK_DIALOG (msg_dialog));
-      gtk_widget_destroy (msg_dialog);
-
-    }
-  else
-    {
-      theme_changed_func (NULL, dialog);
-      gnome_theme_info_register_theme_change ((GFunc) theme_changed_func, dialog);
-      gconf_client_add_dir (client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-      gconf_client_notify_add (client,
-			       KEY_THEME_KEY,
-			       (GConfClientNotifyFunc) &key_theme_changed,
-			       WID ("key_theme_omenu"), NULL, NULL);
-    }
-
-
   g_signal_connect (GTK_TREE_VIEW (WID ("shortcut_treeview")),
 		    "button_press_event",
 		    G_CALLBACK (start_editing_cb), dialog),
@@ -1022,7 +871,7 @@ setup_dialog (GladeXML *dialog)
   reload_key_entries (wm_common_get_current_window_manager(), dialog);
 
   widget = WID ("gnome-keybinding-dialog");
-  capplet_set_icon (widget, "keyboard-shortcut.png");
+  capplet_set_icon (widget, "gnome-settings-keybindings");
   gtk_widget_show (widget);
 
   g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK(cb_dialog_response), NULL);
