@@ -29,7 +29,7 @@
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 
-#include "libgswitchit/gswitchit_xkb_config.h"
+#include "libgswitchit/gswitchit_config.h"
 
 #include "capplet-util.h"
 #include "gconf-property-editor.h"
@@ -40,7 +40,7 @@
 #include "gnome-keyboard-properties-xkb.h"
 
 #define GROUP_SWITCHERS_GROUP "grp"
-#define DEFAULT_GROUP_SWITCH "grp:alt_shift_toggle"
+#define DEFAULT_GROUP_SWITCH "grp:alts_toggle"
 
 static GtkTreeIter current1stLevelIter;
 static const char *current1stLevelId;
@@ -67,8 +67,8 @@ add_variant_to_available_layouts_tree (const XklConfigItemPtr
   GtkTreeIter iter;
   GtkTreeStore *treeStore =
     GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (layoutsTree)));
-  const gchar *fullLayoutName = GSwitchItConfigMergeItems (current1stLevelId,
-							   configItem->name);
+  const gchar *fullLayoutName = GSwitchItKbdConfigMergeItems (current1stLevelId,
+							      configItem->name);
   char *utfVariantName = xci_desc_to_utf8 (configItem);
 
   gtk_tree_store_append (treeStore, &iter, &current1stLevelIter);
@@ -127,7 +127,8 @@ enable_disable_layouts_buttons (GladeXML * dialog)
 
   gtk_widget_set_sensitive (addLayoutBtn,
 			    (nSelectedAvailableLayouts > 0)
-			    && (nSelectedLayouts < maxSelectedLayouts));
+			    && (nSelectedLayouts < maxSelectedLayouts ||
+				maxSelectedLayouts == 0));
   gtk_widget_set_sensitive (delLayoutBtn, nSelectedSelectedLayouts > 0);
 
   if (gtk_tree_selection_get_selected (sSelection, NULL, &iter))
@@ -168,7 +169,7 @@ prepare_selected_layouts_tree (GladeXML * dialog)
   g_signal_connect_swapped (G_OBJECT (selection), "changed",
 			    G_CALLBACK
 			    (enable_disable_layouts_buttons), dialog);
-  maxSelectedLayouts = XklMultipleLayoutsSupported ()? XkbNumKbdGroups : 1;
+  maxSelectedLayouts = XklGetMaxNumGroups();
 }
 
 void
@@ -188,8 +189,8 @@ fill_selected_layouts_tree (GladeXML * dialog)
       char *v1, *utfVisible;
       const char *visible = (char *) curLayout->data;
       gtk_list_store_append (listStore, &iter);
-      if (GSwitchItConfigGetDescriptions (visible, &sl, &l, &sv, &v))
-	visible = GSwitchItConfigFormatFullLayout (l, v);
+      if (GSwitchItKbdConfigGetDescriptions (visible, &sl, &l, &sv, &v))
+	visible = GSwitchItKbdConfigFormatFullLayout (l, v);
       v1 = g_strdup (visible);
       utfVisible = g_locale_to_utf8 (g_strstrip (v1), -1, NULL, NULL, NULL);
       gtk_list_store_set (listStore, &iter,
@@ -279,7 +280,7 @@ add_selected_layout (GtkWidget * button, GladeXML * dialog)
           while (option != NULL)
             {
               char *g, *o;
-              if (GSwitchItConfigSplitItems (option->data, &g, &o))
+              if (GSwitchItKbdConfigSplitItems (option->data, &g, &o))
                 {
                   if (!g_ascii_strcasecmp (g, GROUP_SWITCHERS_GROUP))
                     {
@@ -297,7 +298,7 @@ add_selected_layout (GtkWidget * button, GladeXML * dialog)
                                        &ci ))
 
                 {
-                  const gchar* id = GSwitchItConfigMergeItems (GROUP_SWITCHERS_GROUP, DEFAULT_GROUP_SWITCH);
+                  const gchar* id = GSwitchItKbdConfigMergeItems (GROUP_SWITCHERS_GROUP, DEFAULT_GROUP_SWITCH);
                   optionsList = g_slist_append (optionsList, g_strdup (id));
                   set_selected_options_list (optionsList);
                 }
@@ -391,7 +392,7 @@ void
 register_layouts_gconf_listener (GladeXML * dialog)
 {
   gconf_client_notify_add (gconf_client_get_default (),
-			   GSWITCHIT_CONFIG_XKB_KEY_LAYOUTS,
+			   GSWITCHIT_KBD_CONFIG_KEY_LAYOUTS,
 			   (GConfClientNotifyFunc)
 			   update_layouts_list, dialog, NULL, NULL);
 }
