@@ -1,4 +1,4 @@
-/* $Id: keyboard-drawing.c,v 1.33 2004/07/28 00:04:30 svu Exp $ */
+/* $Id: keyboard-drawing.c,v 1.35 2004/10/24 21:01:24 svu Exp $ */
 /*
  * keyboard-drawing.c: implementation of a gtk+ widget that is a drawing of
  * the keyboard of the default display
@@ -40,6 +40,10 @@ enum
 
 static guint keyboard_drawing_signals[NUM_SIGNALS] = { 0 };
 
+static void
+keyboard_drawing_set_mods(KeyboardDrawing *kbdrawing,
+                          guint            mods);
+
 static gint
 xkb_to_pixmap_coord (KeyboardDrawing *drawing, 
                      gint             n)
@@ -75,6 +79,9 @@ draw_polygon (KeyboardDrawing *drawing,
   GdkPoint *points;
   gboolean filled;
   gint i;
+  
+  if (drawing->pixmap == NULL) 
+    return;
 
   if (fill_color) 
     {
@@ -113,6 +120,9 @@ draw_rectangle (KeyboardDrawing *drawing,
                 gint             xkb_width, 
                 gint             xkb_height)
 {
+  if (drawing->pixmap == NULL)
+    return;
+
   if (angle == 0) 
     {
       GtkStateType state = GTK_WIDGET_STATE (GTK_WIDGET (drawing));
@@ -601,6 +611,9 @@ draw_layout (KeyboardDrawing *drawing,
   gint x_off, y_off;
   gint i;
 
+  if (drawing->pixmap == NULL)
+    return;
+
   if (angle != drawing->angle) 
     {
       if (angle == 0) 
@@ -657,6 +670,52 @@ draw_layout (KeyboardDrawing *drawing,
   gdk_draw_layout (drawing->pixmap, GTK_WIDGET (drawing)->style->text_gc[state], x, y, drawing->layout);
 }
 
+static void 
+draw_key_label_helper (KeyboardDrawing *drawing,
+                       KeySym keysym,
+                       gint angle,
+                       gint groups[], gint g,
+                       gint x, gint y,
+                       gint width, gint height,
+                       gboolean flipped, gint padding, int ypos)
+{
+  gint old_size;
+  gint label_x, label_y, label_max_width;
+
+  if (keysym == 0)
+    return;
+
+  if (groups[g] == drawing->leftgroup) 
+    {
+      rotate_coordinate (x, y, x + padding, 
+                         y + padding + (height - 2 * padding) * (flipped ? (1 - ypos) : ypos) * 4 / 7, 
+                         angle, &label_x, &label_y); 
+      label_max_width = PANGO_SCALE * (width - 2 * padding);
+    } 
+  else if (groups[g] == drawing->rightgroup) 
+    {
+      rotate_coordinate (x, y, x + padding + (width - 2 * padding) * 4 / 7, 
+                         y + padding + (height - 2 * padding) * (flipped ? (1 - ypos) : ypos) * 4 / 7, 
+                         angle, &label_x, &label_y);
+      label_max_width = PANGO_SCALE * ((width - 2 * padding) - (width - 2 * padding) * 4 / 7);
+    } 
+  else
+    return;
+
+  set_key_label_in_layout (drawing, drawing->layout, keysym);
+
+  old_size = pango_font_description_get_size (drawing->font_desc);
+  fit_width (drawing, label_max_width);
+
+  draw_layout (drawing, angle, label_x, label_y, drawing->layout);
+
+  if (pango_font_description_get_size (drawing->font_desc) != old_size) 
+    {
+      pango_font_description_set_size (drawing->font_desc, old_size);
+      pango_layout_set_font_description (drawing->layout, drawing->font_desc);
+    }
+}
+
 static void
 draw_key_label (KeyboardDrawing *drawing,
                 guint            keycode,
@@ -668,7 +727,7 @@ draw_key_label (KeyboardDrawing *drawing,
 {
   gint groups[] = { drawing->leftgroup, drawing->rightgroup };
   gint levels[] = { drawing->bottomlevel, drawing->toplevel };
-  gint x, y, width, height, old_size;
+  gint x, y, width, height;
   gboolean flipped;
   gint padding;
   gint g, l;
@@ -684,51 +743,39 @@ draw_key_label (KeyboardDrawing *drawing,
 
   for (g = 0; g < 2; g++) 
     {
-      gint label_x, label_y, label_max_width;
-
       if (groups[g] < 0 || groups[g] >= XkbKeyNumGroups (drawing->xkb, keycode))
         continue;
 
-      for (l = 0; l < 2; l++) 
+      if (drawing->track_modifiers)
         {
+          uint mods_rtrn;
           KeySym keysym;
 
-          if (levels[l] < 0 || levels[l] >= XkbKeyGroupWidth (drawing->xkb, keycode, groups[g]))
-            continue;
-
-          keysym = XkbKeySymEntry (drawing->xkb, keycode, levels[l], groups[g]);
-
-          if (keysym == 0)
-            continue;
-
-          if (groups[g] == drawing->leftgroup) 
+          if (XkbTranslateKeyCode (drawing->xkb, keycode, 
+                                   XkbBuildCoreState (drawing->mods, groups[g]), 
+                                   &mods_rtrn, &keysym) )
             {
-              rotate_coordinate (x, y, x + padding, 
-                                 y + padding + (height - 2 * padding) * (flipped ? l : (1 - l)) * 4 / 7, 
-                                 angle, &label_x, &label_y); 
-              label_max_width = PANGO_SCALE * (width - 2 * padding);
-            } 
-          else if (groups[g] == drawing->rightgroup) 
+              draw_key_label_helper (drawing, keysym, angle,
+                                     groups, g, 
+                                     x, y, width, height, 
+                                     flipped, padding, 1); /* reverse y order */
+            }
+        }
+      else
+        {
+          for (l = 0; l < 2; l++) 
             {
-              rotate_coordinate (x, y, x + padding + (width - 2 * padding) * 4 / 7, 
-                                 y + padding + (height - 2 * padding) * (flipped ? l : (1 - l)) * 4 / 7, 
-                                 angle, &label_x, &label_y);
-              label_max_width = PANGO_SCALE * ((width - 2 * padding) - (width - 2 * padding) * 4 / 7);
-            } 
-          else
-            continue;
+              KeySym keysym;
 
-          set_key_label_in_layout (drawing, drawing->layout, keysym);
-
-          old_size = pango_font_description_get_size (drawing->font_desc);
-          fit_width (drawing, label_max_width);
-
-          draw_layout (drawing, angle, label_x, label_y, drawing->layout);
-
-          if (pango_font_description_get_size (drawing->font_desc) != old_size) 
-            {
-              pango_font_description_set_size (drawing->font_desc, old_size);
-              pango_layout_set_font_description (drawing->layout, drawing->font_desc);
+              if (levels[l] < 0 || levels[l] >= XkbKeyGroupWidth (drawing->xkb, keycode, groups[g]))
+                continue;
+    
+              keysym = XkbKeySymEntry (drawing->xkb, keycode, levels[l], groups[g]);
+    
+              draw_key_label_helper (drawing, keysym, angle,
+                                     groups, g, 
+                                     x, y, width, height, 
+                                     flipped, padding, 1 - l); /* reverse y order */
             }
         }
     }
@@ -919,6 +966,10 @@ draw_keyboard (KeyboardDrawing *drawing)
   if (drawing->xkb == NULL)
     return;
 
+  printf( "groups: %d, %d\n", drawing->leftgroup, drawing->rightgroup );
+  printf( "levels: %d, %d\n", drawing->bottomlevel, drawing->toplevel );
+  printf( "mods: %d\n", drawing->mods );
+
   g_list_foreach (drawing->keyboard_items, (GFunc) draw_keyboard_item, drawing);
 }
 
@@ -1244,7 +1295,8 @@ xkb_state_notify_event_filter (GdkXEvent       *gdkxev,
                                GdkEvent        *event, 
                                KeyboardDrawing *drawing)
 {
-  gint group_change_mask = XkbGroupStateMask | XkbGroupBaseMask | XkbGroupLatchMask | XkbGroupLockMask;
+#define group_change_mask (XkbGroupStateMask | XkbGroupBaseMask | XkbGroupLatchMask | XkbGroupLockMask)
+#define modifier_change_mask (XkbModifierStateMask | XkbModifierBaseMask | XkbModifierLatchMask | XkbModifierLockMask)
 
   if (((XEvent *) gdkxev)->type == drawing->xkb_event_type) 
     {
@@ -1252,10 +1304,14 @@ xkb_state_notify_event_filter (GdkXEvent       *gdkxev,
       switch (kev->any.xkb_type) 
         {
         case XkbStateNotify:
-          if ((kev->state.changed & group_change_mask) && drawing->track_group) 
+          if (((kev->state.changed & group_change_mask) && drawing->track_group) ||
+              ((kev->state.changed & modifier_change_mask) && drawing->track_modifiers)) 
             {
               free_keys_and_doodads_and_colors (drawing);
-              keyboard_drawing_set_groups (drawing, kev->state.locked_group, -1);
+              if (drawing->track_group)
+                keyboard_drawing_set_groups (drawing, kev->state.group, -1);
+              if (drawing->track_modifiers)
+                keyboard_drawing_set_mods (drawing, kev->state.compat_state);
               drawing->keys = g_new0 (KeyboardDrawingKey, drawing->xkb->max_key_code + 1);
               size_allocate (GTK_WIDGET (drawing), &(GTK_WIDGET (drawing)->allocation), drawing);
 
@@ -1289,8 +1345,17 @@ xkb_state_notify_event_filter (GdkXEvent       *gdkxev,
         case XkbControlsNotify:
         case XkbNamesNotify:
         case XkbNewKeyboardNotify:
-          if (drawing->track_config)
-            keyboard_drawing_set_keyboard (drawing, NULL);
+            {
+              XkbStateRec state;
+              memset (&state, 0, sizeof (state));
+              XkbGetState (drawing->display, XkbUseCoreKbd, &state );
+              if (drawing->track_group) 
+                keyboard_drawing_set_groups (drawing, state.group, -1);
+              if (drawing->track_modifiers) 
+                keyboard_drawing_set_mods (drawing, state.compat_state);
+              if (drawing->track_config)
+                keyboard_drawing_set_keyboard (drawing, NULL);
+            }
           break;
         }
     }
@@ -1342,6 +1407,7 @@ keyboard_drawing_init (KeyboardDrawing *drawing)
   drawing->scale_denominator = 1;
 
   drawing->track_group = 0;
+  drawing->track_modifiers = 0;
   drawing->track_config = 0;
 
   gtk_widget_set_double_buffered (GTK_WIDGET (drawing), FALSE);
@@ -1363,7 +1429,7 @@ keyboard_drawing_init (KeyboardDrawing *drawing)
   mask = (XkbStateNotifyMask|XkbNamesNotifyMask|XkbControlsNotifyMask|XkbIndicatorMapNotifyMask|XkbNewKeyboardNotifyMask);
   XkbSelectEvents (drawing->display, XkbUseCoreKbd, mask, mask);
 
-  mask = XkbGroupStateMask;
+  mask = XkbGroupStateMask|XkbModifierStateMask;
   XkbSelectEventDetails (drawing->display, XkbUseCoreKbd, XkbStateNotify, mask, mask);
 
   mask = (XkbGroupNamesMask|XkbIndicatorNamesMask);
@@ -1441,6 +1507,7 @@ keyboard_drawing_set_groups (KeyboardDrawing *kbdrawing,
                              gint             leftgroup, 
                              gint             rightgroup)
 {
+printf( "set_groups: %d %d \n", leftgroup, rightgroup);
   if (leftgroup != kbdrawing->leftgroup || 
       rightgroup != kbdrawing->rightgroup) 
     {
@@ -1457,6 +1524,7 @@ keyboard_drawing_set_levels (KeyboardDrawing *kbdrawing,
                              gint             bottomlevel, 
                              gint             toplevel)
 {
+printf( "set_levels: %d %d \n", bottomlevel, toplevel);
   if (bottomlevel != kbdrawing->bottomlevel || toplevel != kbdrawing->toplevel) 
     {
       kbdrawing->bottomlevel = bottomlevel;
@@ -1465,6 +1533,18 @@ keyboard_drawing_set_levels (KeyboardDrawing *kbdrawing,
     }
 }
 
+void
+keyboard_drawing_set_mods(KeyboardDrawing *kbdrawing,
+                          guint            mods)
+{
+printf( "set_mods: %d\n", mods);
+  if (mods != kbdrawing->mods) 
+    {
+      kbdrawing->mods = mods;
+      gtk_widget_queue_draw (GTK_WIDGET (kbdrawing));
+    }
+}
+
 /* returns a pixbuf with the keyboard drawing at the current pixel size
  * (which can then be saved to disk, etc) */
 GdkPixbuf *
@@ -1484,15 +1564,16 @@ keyboard_drawing_set_keyboard (KeyboardDrawing      *kbdrawing,
 {
   free_keys_and_doodads_and_colors (kbdrawing);
   XkbFreeKeyboard (kbdrawing->xkb, 0, TRUE);    /* free_all = TRUE */
+  kbdrawing->xkb = NULL;  
 
   if (names)
     kbdrawing->xkb = XkbGetKeyboardByName (kbdrawing->display, XkbUseCoreKbd, names, 0, 
                                            XkbGBN_GeometryMask | XkbGBN_KeyNamesMask | XkbGBN_OtherNamesMask | XkbGBN_ClientSymbolsMask | XkbGBN_IndicatorMapMask, 
                                            FALSE);
   else
-    XkbGetKeyboard (kbdrawing->display,
-                    XkbGBN_GeometryMask | XkbGBN_KeyNamesMask | XkbGBN_OtherNamesMask | XkbGBN_ClientSymbolsMask | XkbGBN_IndicatorMapMask, 
-                    XkbUseCoreKbd);
+    kbdrawing->xkb = XkbGetKeyboard (kbdrawing->display,
+                                     XkbGBN_GeometryMask | XkbGBN_KeyNamesMask | XkbGBN_OtherNamesMask | XkbGBN_ClientSymbolsMask | XkbGBN_IndicatorMapMask, 
+	                              XkbUseCoreKbd);
 
   if (kbdrawing->xkb == NULL)
     return FALSE;
@@ -1553,15 +1634,37 @@ keyboard_drawing_get_compat (KeyboardDrawing * kbdrawing)
 }
 
 void
-keyboard_drawing_set_track_group  (KeyboardDrawing      *kbdrawing, 
-                                   gboolean              enable)
+keyboard_drawing_set_track_group (KeyboardDrawing      *kbdrawing, 
+                                  gboolean              enable)
 {
   if (enable)
-    kbdrawing->track_group = 1;
+    {
+      XkbStateRec state;
+      kbdrawing->track_group = 1;
+      memset (&state, 0, sizeof (state));
+      XkbGetState (kbdrawing->display, XkbUseCoreKbd, &state );
+      keyboard_drawing_set_groups (kbdrawing, state.group, -1);
+    }
   else
     kbdrawing->track_group = 0;
 }
 
+void
+keyboard_drawing_set_track_modifiers (KeyboardDrawing      *kbdrawing, 
+                                  gboolean              enable)
+{
+  if (enable)
+    {
+      XkbStateRec state;
+      kbdrawing->track_modifiers = 1;
+      memset (&state, 0, sizeof (state));
+      XkbGetState (kbdrawing->display, XkbUseCoreKbd, &state );
+      keyboard_drawing_set_mods (kbdrawing, state.compat_state);
+    }
+  else
+    kbdrawing->track_modifiers = 0;
+}
+
 void
 keyboard_drawing_set_track_config (KeyboardDrawing      *kbdrawing, 
                                    gboolean              enable)
